/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
#ifndef SKGREPORTPLUGINWIDGET_H
#define SKGREPORTPLUGINWIDGET_H
/** @file
* This file is Skrooge plugin for bank management.
*
* @author Stephane MANKOWSKI / Guillaume DE BURE
*/
#include "ui_skgreportpluginwidget_base.h"
#include "skgtabpage.h"

#include <QStringList>
#include <QDomDocument>
#include <QTimer>

class SKGDocumentBank;

/**
 * This file is Skrooge plugin to generate report
 */
class SKGReportPluginWidget : public SKGTabPage
{
    Q_OBJECT

public:
    /**
     * Default Constructor
     * @param iDocument the document
     * @param iMinimmumMode the panel will be displayed with minimum widgets
     */
    explicit SKGReportPluginWidget ( SKGDocumentBank* iDocument, bool iMinimmumMode=false );

    /**
     * Default Destructor
     */
    virtual ~SKGReportPluginWidget();

    /**
     * Get the current state
     * MUST BE OVERWRITTEN
     * @return a string containing all information needed to set the same state.
     * Could be an XML stream
     */
    virtual QString getState();

    /**
     * Set the current state
     * MUST BE OVERWRITTEN
     * @param iState must be interpreted to set the state of the widget
     */
    virtual void setState ( const QString& iState );

    /**
     * Get attribute name to save the default state
     * MUST BE OVERWRITTEN
     * @return attribute name to save the default state.
     */
    virtual QString getDefaultStateAttribute();

    /**
     * Get the zoomable widget. MUST BE OVERWRITTEN
     * By Overwritting this function, you don't have to implement following ones.
     * @return the tree view.
     */
    virtual QWidget* zoomableWidget() const;

    /**
     * Create an historized table.
     * Each column is the sum off previous ones
     * @param iTable table
     */
    static SKGStringListList getHistorizedTable ( const SKGStringListList& iTable );

public Q_SLOTS:
    /**
     * Refresh the panel
     */
    void refresh();

    /**
    * data are modified
    * @param iTableName table name
    * @param iIdTransaction the id of the transaction for direct modifications of the table (update of modify objects is enough)
    *or 0 in case of modifications by impact (full table must be refreshed)
    */
    virtual void dataModified ( const QString& iTableName, int iIdTransaction );

private Q_SLOTS:
    void onDoubleClick ( int row, int column );
    void onOpenReport();
    void onSelectionChanged();
    void onBtnModeClicked();
    void refreshActivated();
    void onOneLevelMore();
    void onOneLevelLess();

private:
    Q_DISABLE_COPY ( SKGReportPluginWidget );

    int getCurrentMode();
    void setCurrentMode ( int iMode );

    QString getConsolidatedWhereClause ( QString* oWhereClausForPreviousData=NULL, QString*  oWhereClausForForecastData=NULL );
    void getWhereClauseAndTitleForSelection ( int row, int column, QString& oWc, QString& oTitle );

    Ui::skgreportplugin_base ui;
    QString previousParametersUsed;
    QStringList AttsForColumns;
    QStringList AttsForLines;
    QString operationWhereClause;
    QDomDocument lastState;
    QAction* openReportAction;
    QTimer* timer;
    int mode;
    int nbLevelLines;
    int nbLevelColumns;
};

#endif // SKGREPORTPLUGINWIDGET_H
